/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * http.h
 *                     
 * Author : Adam Dunkels <adam@sics.se>                               
 *
 * CHANGELOG: this file has been modified by Sergio Perez Alcaсiz <serpeal@upvnet.upv.es> 
 *            Departamento de Informбtica de Sistemas y Computadores          
 *            Universidad Politйcnica de Valencia                             
 *            Valencia (Spain)    
 *            Date: March 2003                                          
 *
 */

#ifndef __HTTPD_H__
#define __HTTPD_H__

#include "lwip/def.h"
#include "fsdata.h"
#include "stm32f4xx.h"
#include "stdbool.h"

#define SET_FWINVALID_FLAG()    do {\
                                    fInvalidFw = true;\
                                    RTC_WriteBackupRegister(RTC_BKP_DR7, fInvalidFw);\
                                } while(0)

#define CLEAR_FWINVALID_FLAG()   do {\
                                    fInvalidFw = false;\
                                    RTC_WriteBackupRegister(RTC_BKP_DR7, fInvalidFw);\
                                } while(0)

#define SET_FWUPDATED_FLAG()    do {\
                                    RTC_WriteBackupRegister(RTC_BKP_DR3, 0x01);\
                                } while(0)

typedef enum 
{
  LoginPage = 0,
  FileUploadPage,
  UploadDonePage,
  UploadErrorPage,
  ResetDonePage
  	
}htmlpageState;

struct fs_file {
  char *data;
  int len;
};

void IAP_httpd_init(void);

uint16_t Finder(char* data, uint16_t len);

uint32_t ReturnFlashWriteAddress(void);

bool GetFileName(char *inStr, char *fileName, uint8_t *fileNameLen);

uint8_t update_timeout;

#endif /* __HTTPD_H__ */
